define([
  '../../core/Class'
], function(Class) {

  var rootAbstractDomContext;

  var AbstractDomContext = Class.extend('AbstractDomContext', {

    merge: function(first, second) {
      var l = second.length,
        i = first.length,
        j = 0;

      while (second[j] !== undefined) {
        first[i++] = second[j++];
      }

      first.length = i;

      return first;
    },

    // results is for internal usage only
    makeArray: function(arr, results) {
      var ret = results || [];

      if (arr !== null) {
        if (isArraylike(Object(arr))) {
          jQuery.merge(ret, typeof arr === 'string' ? [arr] : arr);
        }
        else {
          core_push.call(ret, arr);
        }
      }

      return ret;
    }

  }, {

    init: function(selector, context, rootAbstractDomContext) {

      this.selector = '';
      this.length = 0;

      // HANDLE: $(''), $(null), $(undefined), $(false)
      if (!selector) {
        return this;
      }

      if (selector.nodeType) {
        this.context = this[0] = selector;
        this.length = 1;
        return this;
      }

      if (selector.selector !== undefined) {
        this.selector = selector.selector;
        this.context = selector.context;
      }

      return AbstractDomContext.makeArray(selector, this);
    },

    // The number of elements contained in the matched element set
    size: function() {
      return this.length;
    },

    toArray: function() {
      return core_slice.call(this);
    },

    // Get the Nth element in the matched element set OR
    // Get the whole matched element set as a clean array
    get: function(num) {
      return num === null ?

        // Return a 'clean' array
        this.toArray() :

        // Return just the object
        (num < 0 ? this[this.length + num] : this[num]);
    },

    // Take an array of elements and push it onto the stack
    // (returning the new matched element set)
    pushStack: function(elems) {

      // Build a new AbstractDomContext matched element set
      var ret = AbstractDomContext.merge(new this.constructor(), elems);

      // Add the old object onto the stack (as a reference)
      ret.prevObject = this;
      ret.context = this.context;

      // Return the newly-formed element set
      return ret;
    },

    find: function(selector) {
      var i, j, ret, partial,
        len = this.length;

      ret = [];
      for (i = 0; i < len; i++) {
        ret.push.apply(ret, this.constructor.find(selector, this[i]));
        /*partial = query(selector, this[i], ret);
        for (j = 0 ; j < partial.length ; j++) {
          ret.push()
        }*/
      }

      // Needed because $(selector, context) becomes $(context).find(selector)
      ret = this.pushStack(ret);
      ret.selector = (this.selector ? this.selector + ' ' : '') + selector;
      return ret;
    }

  });

  return AbstractDomContext;
});
