define([
    '../../core/Initializer',
    '../../mixins/ObjectCapsule',
    '../../mixins/TypedCapsule',
    './Url',
    '../error/MissingArgumentError',
    '../error/MissingImplementationError'
  ],
  function (Initializer, ObjectCapsule, TypedCapsule, Url, MissingArgumentError, MissingImplementationError) {
    'use strict';

    var statics = {
        METHOD_GET   : 'GET',
        METHOD_POST  : 'POST',
        METHOD_PUT   : 'PUT',
        METHOD_DELETE: 'DELETE'
      };

    return Initializer.extend('AbstractWebServiceAdapter', [
        new TypedCapsule('baseUrl', Url),
        new ObjectCapsule('options')
      ],
      statics,
      {
        init: function (config) {
          if(!config) {
            throw new MissingArgumentError('config');
          }
          if(!config.baseUrl) {
            throw new MissingArgumentError('config.baseUrl');
          }
          return this._super(config);
        },

        initBaseUrl: function(config) {
          this.setBaseUrl(config.baseUrl);
        },

        initOptions: function(config) {
          this.setOptions(config.options || {});
        },

        send: function (/*method, relativeUrl, options*/) {
          throw new MissingImplementationError(this.constructor, 'send');
        },

        get: function (relativeUrl, options) {
          return this.send(statics.METHOD_GET, relativeUrl, options);
        },

        post: function (relativeUrl, options) {
          return this.send(statics.METHOD_POST, relativeUrl, options);
        },

        put: function (relativeUrl, options) {
          return this.send(statics.METHOD_PUT, relativeUrl, options);
        },

        'delete': function (relativeUrl, options) {
          return this.send(statics.METHOD_DELETE, relativeUrl, options);
        }
      });
  }
);
