
define(['./Capsule'], function(Capsule) {
  'use strict';

  if(!('bind' in Function.prototype)) {
    throw new Error('Function.prototype.bind is not defined');
  }

  var toString = Object.prototype.toString;

  function isArray(value) {
    return toString.call(value) === '[object Array]';
  }

  function singularize(name) {
    return name.replace(/s$/, '');
  }

  function makeMethodName(options, fieldName, methodNamePrefix, methodNameSuffix) {
    return Capsule.makeMethodName(options, singularize(fieldName), methodNamePrefix, methodNameSuffix);
  }

  function InvalidArrayIndexError(message) {
    this.name = 'InvalidArrayIndex';
    this.message = (message || '');
  }
  InvalidArrayIndexError.prototype = Error.prototype;

  function InvalidArrayError(message) {
    this.name = 'InvalidArray';
    this.message = (message || '');
  }
  InvalidArrayError.prototype = Error.prototype;

  var ArrayCapsule = function ArrayCapsule(fieldName, options) {
    var methods = {},
      methodNames = {
        'set'          : Capsule.makeMethodName(options, fieldName, 'set'),
        'add'          : makeMethodName(options, fieldName, 'add'),
        'removeByIndex': makeMethodName(options, fieldName, 'remove', 'ByIndex')
      },
      capsuleOptions = {
        defaultValue: options && options.hasOwnProperty('defaultValue') ? options.defaultValue : [],
        methodNames : options && options.hasOwnProperty('methodNames') ? options.methodNames : null
      };

    // setXXX
    // Note: copy the values into a new empty array
    methods[methodNames.set] = function setValues(values) {
      if(!isArray(values)) {
        throw new InvalidArrayError(this + '.' + methodNames.add + '(): expecting argument to be an array, ' + toString.call(values) + ' given');
      }
      this[fieldName] = [];
      values.forEach(this[methodNames.add].bind(this));
      return this;
    };

    // addXXX
    methods[methodNames.add] = function addValue(value) {
      if(!Capsule.validate(options, value)) {
        throw new Capsule.InvalidValueError(this + '.' + methodNames.add + '(): trying to add an invalid value "' + value + '" (' + typeof value + ')');
      }
      if(!isArray(this[fieldName])) {
        throw new InvalidArrayError(this + '.' + methodNames.add + '(): trying to add an value to a non-array: "' + this[fieldName] + '" (' + typeof this[fieldName] + ')');
      }
      this[fieldName].push(value);
      return this;
    };

    // removeXXXByIndex
    methods[methodNames.removeByIndex] = function removeValueByIndex(index) {
      if(index < 0) {
        throw new InvalidArrayIndexError(this + '.' + methodNames.removeByIndex + '(): trying to remove an element at negative index ' + index);
      }
      if(index >= this[fieldName].length) {
        throw new InvalidArrayIndexError(this + '.' + methodNames.removeByIndex + '(): trying to remove an element at index ' + index + ', while array length is ' + this[fieldName].length);
      }
      this[fieldName].splice(index, 1);
      return this;
    };

    // Returns the setter, wrapped into a Class extending Capsule
    return new Capsule(fieldName, capsuleOptions).extend('ArrayCapsule', methods);
  };

  ArrayCapsule.makeMethodName = makeMethodName;
  ArrayCapsule.InvalidArrayIndexError = InvalidArrayIndexError;

  return ArrayCapsule;
});
