define([
    'jssl/core/Initializer',
    'jssl/mixins/ObjectCapsule',
    'jssl/mixins/TypedCapsule',
    'jssl/webservice/AbstractWebServiceAdapter',
    'jssl/webservice/JQueryWebServiceAdapter',
    'jssl/error/MissingArgumentError',
    './VersionInfo',
    './User'
  ],
  function (Initializer, ObjectCapsule, TypedCapsule, AbstractWebServiceAdapter,
    JQueryWebServiceAdapter, MissingArgumentError, VersionInfo, User) {

    'use strict';

    return Initializer.extend('Session', [
        new ObjectCapsule('api'),
        new TypedCapsule('xhrAdapter' , AbstractWebServiceAdapter),
        new TypedCapsule('versionInfo', VersionInfo),
        new TypedCapsule('user'       , User)
      ], {

        init: function(api) {
          if(!api) {
            throw new MissingArgumentError('api');
          }
          if(!api.url) {
            throw new MissingArgumentError('api.url');
          }
          if(!api.version) {
            throw new MissingArgumentError('api.version');
          }
          if(!api.adapter) {
            throw new MissingArgumentError('api.adapter');
          }
          this._super(api);
        },

        initVersionInfo: function(api) {

          if(!this._initVersionInfoRequest) {
            this.initXhrAdapter(api);
            this._initVersionInfoRequest = this.getXhrAdapter().get('/versions')
              .then(function (data) {
                console.log('Version info', data);
              })
              .fail(function () {
                console.log('Version info', arguments);
              });
          }

          return this._initVersionInfoRequest;
        },

        initXhrAdapter: function(api) {
          if(!this.getXhrAdapter()) {
            this.setXhrAdapter(new JQueryWebServiceAdapter({
              baseUrl: api.url,
              options: api.adapter.options || {}
            }));
          }
        },

        authenticate: function (credentials) {

          if(!credentials.organization) {
            throw new MissingArgumentError('credentials.organization');
          }
          if(!credentials.username) {
            throw new MissingArgumentError('credentials.username');
          }
          if(!credentials.password) {
            throw new MissingArgumentError('credentials.password');
          }

          return this.getXhrAdapter().post('/sessions');
        }

      });
  }
);
