/*global Document*/

define(['./XMLHttpRequest'], function (XMLHttpRequest) {
  'use strict';

  return function loadXMLDocuments(directory, files, check) {

    var result = {};
    QUnit.asyncTest('Load test XML documents', function () {
      var remaining = files.length;
      files.forEach(function(documentName) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', directory + '/' + documentName, true);
        xhr.onreadystatechange = function () {
          if (this.readyState === XMLHttpRequest.DONE) {
            QUnit.strictEqual(this.status, 200, documentName + ' - HTTP status code should be 200 OK');

            // 1. XML
            if(this.responseXML) {
              QUnit.ok(this.responseXML instanceof Document, documentName + ' - responseXML should be a Document');
              QUnit.ok(this.responseXML.documentElement, documentName + ' - responseXML should have a documentElement field');
              result[documentName] = this.responseXML;
            }
            // 2. HTML (create an iframe and populates it with responseText)
            else {
              var iframe = document.createElement('iframe');
              iframe.sandbox = 'allow-scripts allow-same-origin';
              document.getElementById('qunit-fixture').appendChild(iframe);
              /*
              if(iframe.webNavigation) {
                iframe.webNavigation.allowAuth = false;
                iframe.webNavigation.allowImages = false;
                iframe.webNavigation.allowJavascript = false;
                iframe.webNavigation.allowMetaRedirects = true;
                iframe.webNavigation.allowPlugins = false;
                iframe.webNavigation.allowSubframes = false;
              }
              */
              (iframe.contentDocument || iframe.contentWindow.document).write(this.responseText);
              QUnit.ok(iframe.contentDocument instanceof iframe.contentWindow.HTMLDocument, documentName + ' - iframe.contentDocument should be a HTMLDocument');
              QUnit.ok(iframe.contentDocument.documentElement, documentName + ' - iframe.contentDocument should have a documentElement field');
              result[documentName] = iframe.contentDocument;
            }

            remaining--;
            if(remaining === 0) {
              QUnit.start();
            }
          }
        };
        xhr.send();
      });
    });

    if(check) {
      QUnit.test('Check loaded documents', function () {
        files.forEach(function(documentName) {
          QUnit.ok(result[documentName], documentName + ' exists in result object');
          QUnit.ok(result[documentName] instanceof Document, documentName + ' is a Document');
        });
      });
    }

    return result;
  };

});
