define([
  'jssl/error/JsslError',
  'jssl/error/MissingArgumentError',
  'jssl/error/MissingImplementationError'
],
function(JsslError, MissingArgumentError, MissingImplementationError) {

  var errorClasses = [], i;
  for(i = 0 ; i < arguments.length ; i++) {
    errorClasses[i] = arguments[i];
  }

  QUnit.module('jssl/error');


  /*
  QUnit.test('Error instances inheritance from Error', function () {
    errorClasses.forEach(function (errorClass) {
      QUnit.ok(new errorClass() instanceof Error, 'new ' + errorClass.name + '() should be an instance of Error');
    });
  });

  QUnit.test('Error instances inheritance from JsslError', function () {
    errorClasses.filter(function (c) { return c !== JsslError}).forEach(function (errorClass) {
      QUnit.ok(new errorClass() instanceof JsslError, 'new ' + errorClass.name + '() should be an instance of JsslError');
    });
  });

  QUnit.test('Error instances inheritance from their constructor', function () {
    errorClasses.forEach(function (errorClass) {
      QUnit.ok(new errorClass() instanceof errorClass, 'new ' + errorClass.name + '() should be an instance of ' + errorClass.name);
    });
  });

  QUnit.test('Compatibility with QUnit\'s throws()', function () {
    errorClasses.forEach(function (errorClass) {
    QUnit.throws(function() { throw new errorClass(); }, errorClass, 'throw new ' + errorClass.name + '() should be caught by QUnit as a ' + errorClass.name + ' error');
    });
  });

  QUnit.test('Stacktrace', function () {
    errorClasses.forEach(function (errorClass) {
      try {
        throw new errorClass('test');
      }
      catch(e) {
        var stack = e.stack.split('\n').splice(0, 2).join('\n'),
          expected = [
              errorClass.name + ': ' + e.message,
              '    at ' + window.location.href.replace(/\/[^\/]*$/, '') + '/jssl/ErrorsTest.js:42:15'
            ].join('\n');
        QUnit.strictEqual(stack, expected, 'Stack trace of new ' + errorClass.name + '() should start with: \n' + expected);
      }
    });
  });
  */

  QUnit.test('Error instances inheritance from Error', function () {
    errorClasses.forEach(function (errorClass) {
      QUnit.ok(new errorClass() instanceof Error, 'new ' + errorClass.name + '() should be an instance of Error');
    });
  });

  QUnit.test('Error instances inheritance from JsslError', function () {
    errorClasses.filter(function (c) { return c !== JsslError; }).forEach(function (errorClass) {
      QUnit.ok(new errorClass() instanceof JsslError, 'new ' + errorClass.name + '() should be an instance of MyModuleError');
    });
  });

  QUnit.test('Error instances inheritance from their constructor', function () {
    errorClasses.forEach(function (errorClass) {
      QUnit.ok(new errorClass() instanceof errorClass, 'new ' + errorClass.name + '() should be an instance of ' + errorClass.name);
    });
  });

  QUnit.test('Compatibility with QUnit\'s throws()', function () {
    errorClasses.forEach(function (errorClass) {
    QUnit.throws(function() { throw new errorClass(); }, errorClass, 'throw new ' + errorClass.name + '() should be caught by QUnit as a ' + errorClass.name + ' error');
    });
  });

  QUnit.asyncTest('Stacktrace', function () {
    var errorMessage = new Date().getTime(),
      nbTests = 0,
      nbLines = 7,
      headerLine = false;
    errorClasses.forEach(function (errorClass) {
      function testErrorClass1() {
        throw new errorClass(errorMessage);
      }
      function testErrorClass2() {
        testErrorClass1();
      }
      function testErrorClass3() {
        testErrorClass2();
      }
      function testErrorClass4() {
        testErrorClass3();
      }
      function testErrorClass5() {
        testErrorClass4();
      }
      setTimeout(function() {
        try {
          testErrorClass5();
        }
        catch(e) {
          QUnit.strictEqual(typeof e.stack, 'string', 'e.stack should be a String');
          var stack = e.stack.split('\n'), i = 0;
          QUnit.strictEqual(stack.length, nbLines, 'Stack trace should contain ' + nbLines + ' lines');

          // Line #1 - if present
          if(stack[0].indexOf(e.name) !== -1) {
            headerLine = true;
            QUnit.ok(stack[0].indexOf(errorMessage) !== -1, 'Stack trace should contain "' + errorMessage + '" at line #1, got: "' + stack[0] + '"');
            QUnit.strictEqual(stack[0], errorClass.name + ': ' + e.message, 'Stack trace of new ' + errorClass.name + '() should start with "' + errorClass.name + ': ' + errorMessage + '"');
          }

          // Lines #2 to #6
          for(i = 1; i < nbLines-1 ; i++) {
            QUnit.ok(stack[headerLine ? i : i-1].indexOf('ErrorsTest.js') !== -1, 'Stack trace should contain "' + 'ErrorsTest.js' + '" at line #' + (i+1) + ', got: "' + stack[headerLine ? i : i-1] + '"');
            QUnit.ok(stack[headerLine ? i : i-1].indexOf('testErrorClass' + i) !== -1, 'Stack trace should contain "testErrorClass' + i + '" at line #' + (i+1) + ', got: "' + stack[headerLine ? i : i-1] + '"');
          }
        }
        nbTests++;
        if(nbTests === 3) {
          QUnit.start();
        }
      }, 0);
    });
  });
});
