define(['../../../common/loadXMLDocuments', 'require'], function (loadXMLDocuments, require) {
  'use strict';

  return function(DomAdapterClass) {

    var doc1 = 'document.html';
    var doc2 = 'document.xml';
    var documents = loadXMLDocuments(require.toUrl('./files'), [doc1, doc2], require.debug);

    QUnit.test('Context', function () {

      var doc = documents[doc2];
      var catalog = new DomAdapterClass().find('catalog', doc);

      QUnit.ok('length' in catalog, 'Context must have a "length" property');
      QUnit.strictEqual(catalog.length, 1, 'Context length must be 1 for "catalog"');
      QUnit.strictEqual(catalog.get(0), doc.documentElement, 'Context .get(0) must return original DOM node for "catalog"');

      var discs = catalog.find('cd');
      QUnit.strictEqual(discs.length, 26, 'Context length must be 26 for "catalog cd"');
    });

    QUnit.test('Children selector', function () {
      var doc = documents[doc2];

      QUnit.strictEqual(new DomAdapterClass(doc).find('catalog > cd').length             , 26, 'Context length must be 26 for new ' + DomAdapterClass.name + '(responseXML).find("catalog > cd")');
      QUnit.strictEqual(new DomAdapterClass(doc).find('catalog> cd' ).length             , 26, 'Context length must be 26 for new ' + DomAdapterClass.name + '(responseXML).find("catalog> cd")');
      QUnit.strictEqual(new DomAdapterClass(doc).find('catalog >cd' ).length             , 26, 'Context length must be 26 for new ' + DomAdapterClass.name + '(responseXML).find("catalog >cd")');
      QUnit.strictEqual(new DomAdapterClass(doc).find('catalog>cd'  ).length             , 26, 'Context length must be 26 for new ' + DomAdapterClass.name + '(responseXML).find("catalog>cd")');
      QUnit.strictEqual(new DomAdapterClass(doc).find('catalog'     ).find('> cd').length, 26, 'Context length must be 26 for new ' + DomAdapterClass.name + '(responseXML).find("catalog").find("> cd")');
      QUnit.strictEqual(new DomAdapterClass(doc).find('catalog'     ).find('>cd').length , 26, 'Context length must be 26 for new ' + DomAdapterClass.name + '(responseXML).find("catalog").find(">cd")');

      QUnit.strictEqual(new DomAdapterClass().find('catalog > cd', doc).length             , 26, 'Context length must be 26 for new ' + DomAdapterClass.name + '().find("catalog > cd", responseXML)');
      QUnit.strictEqual(new DomAdapterClass().find('catalog> cd' , doc).length             , 26, 'Context length must be 26 for new ' + DomAdapterClass.name + '().find("catalog> cd", responseXML)');
      QUnit.strictEqual(new DomAdapterClass().find('catalog >cd' , doc).length             , 26, 'Context length must be 26 for new ' + DomAdapterClass.name + '().find("catalog >cd", responseXML)');
      QUnit.strictEqual(new DomAdapterClass().find('catalog>cd'  , doc).length             , 26, 'Context length must be 26 for new ' + DomAdapterClass.name + '().find("catalog>cd", responseXML)');
      QUnit.strictEqual(new DomAdapterClass().find('catalog'     , doc).find('> cd').length, 26, 'Context length must be 26 for new ' + DomAdapterClass.name + '().find("catalog").find("> cd", responseXML)');
      QUnit.strictEqual(new DomAdapterClass().find('catalog'     , doc).find('>cd').length , 26, 'Context length must be 26 for new ' + DomAdapterClass.name + '().find("catalog").find(">cd", responseXML)');
    });

    QUnit.test('Selectors (HTML Document)', function () {

      var doc = documents[doc1];

      var selectors = {
        'body': 1,
        'div': 51,
        'body div': 51,
        'div p': 140,
        'div > p': 134,
        'div + p': 22,
        'div ~ p': 183,
        'div[class^=exa][class$=mple]': 43,
        'div p a': 12,
        'div, p, a': 671,
        '.note': 14,
        'div.example': 43,
        'ul .tocline2': 12,
        'div.example, div.note': 44,
        '#title': 1,
        'h1#title': 1,
        'div #title': 1,
        'ul.toc li.tocline2': 12,
        'ul.toc > li.tocline2': 12,
        'h1#title + div > p': 0,
        'h1[id]:contains(Selectors)': 1,
        'a[href][lang][class]': 1,
        'div[class]': 51,
        'div[class=example]': 43,
        'div[class^=exa]': 43,
        'div[class$=mple]': 43,
        'div[class*=e]': 50,
        'div[class|=dialog]': 0,
        'div[class!=made_up]': 51,
        'div[class~=example]': 43,
        'div:not(.example)': 8,
        'p:contains(selectors)': 54,
        'p:nth-child(even)': 158,
        'p:nth-child(2n)': 158,
        'p:nth-child(odd)': 166,
        'p:nth-child(2n+1)': 166,
        'p:nth-child(n)': 324,
        'p:only-child': 3,
        'p:last-child': 19,
        'p:first-child': 54
      };

      for(var selector in selectors) {
        var adapter = new DomAdapterClass(doc);
        if(selectors.hasOwnProperty(selector)) {
          var expected = selectors[selector];
          QUnit.strictEqual(adapter.find(selector).length, expected, 'Selector "' + selector + '" should match ' + expected + ' elements');
        }
      }
    });

    QUnit.test('Selectors (XML Document)', function () {

      var doc = documents[doc2];

      var selectors = {
        '*': 183,
        'cd': 26,
        '.new': 5,
        '#hide-your-heart': 1,
        'cd title': 26,
        'cd > title': 26,
        'artist + country': 26,
        'cd[class]': 16,
        'cd[class=new]': 3,
        'cd[class~=new]': 5,
        'cd[class|=genre]': 3,
        ':first-child': 28,
        'cd[class^=new]': 8,
        'cd[class$=ish]': 6,
        'cd[class*=ew]': 9,
        'title ~ country': 26,
        ':root': 1,
        ':last-child': 28,
        ':only-child': 1,
        //'cd:nth-child(5n + 1)': 6,
        //'cd:nth-last-child(5n + 1)': 6,
        'cd:first-of-type': 1,
        'cd:last-of-type': 1,
        ':only-of-type': 157,
        //'cd:nth-of-type(4n + 1)': 7,
        //'cd:nth-last-of-type(4n + 1)': 7,
        //':empty': -1,
        ':not(.new)': 178,
        ':contains("of")': 7
      };

      for(var selector in selectors) {
        var adapter = new DomAdapterClass(doc);
        if(selectors.hasOwnProperty(selector)) {
          var expected = selectors[selector];
          QUnit.strictEqual(adapter.find(selector).length, expected, 'Selector "' + selector + '" should match ' + expected + ' elements');
        }
      }
    });

  };

});
