define(['jssl/core/ClassDebug', 'jssl/core/Initializer', 'jssl/mixins/ArrayCapsule'],
    function(Class, Initializer, ArrayCapsule) {

  QUnit.module('jssl/mixins/ArrayCapsule');

  QUnit.test('Instanciation', function() {

    var Shop, shop;

    Shop = ArrayCapsule('employees').extend('Shop', {});
    shop = new Shop();
    QUnit.ok('initEmployees'         in shop, 'new Shop() should have a method "initEmployees"');
    QUnit.ok('getEmployees'          in shop, 'new Shop() should have a method "getEmployees"');
    QUnit.ok('setEmployees'          in shop, 'new Shop() should have a method "setEmployees"');
    QUnit.ok('addEmployee'           in shop, 'new Shop() should have a method "addEmployee"');
    QUnit.ok('removeEmployeeByIndex' in shop, 'new Shop() should have a method "removeEmployeeByIndex"');
    QUnit.strictEqual(typeof shop.initEmployees        , 'function', 'new Shop().initEmployees should be a function');
    QUnit.strictEqual(typeof shop.getEmployees         , 'function', 'new Shop().getEmployees should be a function');
    QUnit.strictEqual(typeof shop.setEmployees         , 'function', 'new Shop().setEmployees should be a function');
    QUnit.strictEqual(typeof shop.addEmployee          , 'function', 'new Shop().addEmployee should be a function');
    QUnit.strictEqual(typeof shop.removeEmployeeByIndex, 'function', 'new Shop().removeEmployeeByIndex should be a function');

    Shop = Initializer.extend('Shop', [ArrayCapsule('employees')], {});
    shop = new Shop();
    QUnit.ok('initEmployees'         in shop, 'new Shop() should have a method "initEmployees"');
    QUnit.ok('getEmployees'          in shop, 'new Shop() should have a method "getEmployees"');
    QUnit.ok('setEmployees'          in shop, 'new Shop() should have a method "setEmployees"');
    QUnit.ok('addEmployee'           in shop, 'new Shop() should have a method "addEmployee"');
    QUnit.ok('removeEmployeeByIndex' in shop, 'new Shop() should have a method "removeEmployeeByIndex"');
    QUnit.strictEqual(typeof shop.initEmployees        , 'function', 'new Shop().initEmployees should be a function');
    QUnit.strictEqual(typeof shop.getEmployees         , 'function', 'new Shop().getEmployees should be a function');
    QUnit.strictEqual(typeof shop.setEmployees         , 'function', 'new Shop().setEmployees should be a function');
    QUnit.strictEqual(typeof shop.addEmployee          , 'function', 'new Shop().addEmployee should be a function');
    QUnit.strictEqual(typeof shop.removeEmployeeByIndex, 'function', 'new Shop().removeEmployeeByIndex should be a function');
  });

  QUnit.test('initXXX', function() {
    var Shop = ArrayCapsule('employees').extend('Shop', {});
    var shop = new Shop();
    QUnit.ok('employees' in shop, 'new Shop() should have a field "employees"');
    QUnit.deepEqual(shop.employees, [], 'new Shop().employees should equal []');
  });

  QUnit.test('getXXX', function() {
    var Shop = ArrayCapsule('employees').extend('Shop', {});
    var shop = new Shop();
    QUnit.deepEqual(shop.employees, [], 'new Shop().employees should equal []');
    QUnit.deepEqual(shop.getEmployees(), [], 'new Shop().getEmployees() should equal []');
    shop.employees = [1,2,3];
    QUnit.deepEqual(shop.getEmployees(), [1,2,3], 'shop.getEmployees() should equal [1,2,3]');
  });

  QUnit.test('setXXX', function() {
    var Shop = ArrayCapsule('employees').extend('Shop', {});
    var shop = new Shop();
    QUnit.deepEqual(shop.getEmployees(), [], 'new Shop().employees should equal []');
    shop.setEmployees(["Jimmy"]);
    QUnit.deepEqual(shop.getEmployees(), ["Jimmy"], 'shop.getEmployees() should equal ["Jimmy"]');
    QUnit.strictEqual(shop.setEmployees(["Brian"]), shop, 'shop.setEmployees() should return shop');
    QUnit.deepEqual(shop.getEmployees(), ["Brian"], 'shop.getEmployees() should equal "Brian"');
  });

  QUnit.test('addXXX', function() {
    var Shop = ArrayCapsule('employees').extend('Shop', {});
    var shop = new Shop();
    QUnit.deepEqual(shop.getEmployees(), [], 'new Shop().employees should equal []');
    shop.addEmployee("Jimmy");
    QUnit.deepEqual(shop.getEmployees(), ["Jimmy"], 'shop.addEmployee("Jimmy").getEmployees() should equal ["Jimmy"]');
    QUnit.strictEqual(shop.addEmployee("Brian"), shop, 'shop.addEmployee("Jimmy").addEmployee("Brian") should return shop');
    QUnit.deepEqual(shop.getEmployees(), ["Jimmy","Brian"], 'shop.addEmployee("Jimmy").addEmployee("Brian").getEmployees() should equal ["Jimmy","Brian"]');
  });


  QUnit.test('removeXXXByIndex', function() {
    var Shop = ArrayCapsule('employees').extend('Shop', {});
    var shop = new Shop();
    QUnit.deepEqual(shop.getEmployees(), [], 'new Shop().employees should equal []');
    shop.addEmployee("Jimmy");
    shop.addEmployee("Brian");
    shop.addEmployee("Timmy");
    shop.removeEmployeeByIndex(0);
    QUnit.deepEqual(shop.getEmployees(), ["Brian","Timmy"], 'shop.addEmployee(...).removeEmployeeByIndex() should equal ["Brian","Timmy"]');
    QUnit.strictEqual(shop.removeEmployeeByIndex(0), shop, 'shop.addEmployee(...).removeEmployeeByIndex() should return shop');
    QUnit.raises(function() { shop.removeEmployeeByIndex(-1); }, ArrayCapsule.InvalidArrayIndexError, 'shop.removeEmployeeByIndex(-1) should raise a InvalidArrayIndexError');
    QUnit.raises(function() { shop.removeEmployeeByIndex( 1); }, ArrayCapsule.InvalidArrayIndexError, 'shop.removeEmployeeByIndex(length+) should raise a InvalidArrayIndexError');
  });

  QUnit.test('Options: default value', function() {

    var validValues = [undefined, null, [], ['ok']],
        i,
        Shop,
        shop;

    // ArrayCapsule as a parent
    for(i = 0 ; i < validValues.length ; i++) {
      Shop = ArrayCapsule('employees', {defaultValue:validValues[i]}).extend('Shop', {});
      shop = new Shop();
      QUnit.strictEqual(shop.employees, validValues[i], '[parent] new Shop().employees should have default value ' + validValues[i]);
    }

    // Initializer as a parent and ArrayCapsule as a mixin
    for(i = 0 ; i < validValues.length ; i++) {
      Shop = Initializer.extend('Shop', [ArrayCapsule('employees', {defaultValue:validValues[i]})], {});
      shop = new Shop();
      QUnit.strictEqual(shop.employees, validValues[i], '[mixin] new Shop().employees should have default value ' + validValues[i]);
    }
  });

  QUnit.test('Options: method names', function() {
    var Shop, shop;
    {
      Shop = ArrayCapsule('employees', {
        methodNames: {
          'get'          : 'getSlaves',
          'set'          : 'setSlaves',
          'init'         : 'initSlaves',
          'add'          : 'addSlave',
          'removeByIndex': 'removeSlaveByIndex'
        }
      }).extend('Shop', {});
      shop = new Shop();
      QUnit.ok(!('initEmployees'         in shop), 'new Shop() should not have a method "initEmployees"');
      QUnit.ok(!('getEmployees'          in shop), 'new Shop() should not have a method "getEmployees"');
      QUnit.ok(!('setEmployees'          in shop), 'new Shop() should not have a method "setEmployees"');
      QUnit.ok(!('addEmployee'           in shop), 'new Shop() should not have a method "addEmployee"');
      QUnit.ok(!('removeEmployeeByIndex' in shop), 'new Shop() should not have a method "removeEmployeeByIndex"');
      QUnit.ok('initSlaves'         in shop, 'new Shop() should  have a method "initSlave"');
      QUnit.ok('getSlaves'          in shop, 'new Shop() should have a method "getSlave"');
      QUnit.ok('setSlaves'          in shop, 'new Shop() should have a method "setSlave"');
      QUnit.ok('addSlave'           in shop, 'new Shop() should have a method "addSlaves"');
      QUnit.ok('removeSlaveByIndex' in shop, 'new Shop() should have a method "removeSlaveByIndex"');
      QUnit.strictEqual(typeof shop.initSlaves        , 'function', 'new Shop().initSlaves should be a function');
      QUnit.strictEqual(typeof shop.getSlaves         , 'function', 'new Shop().getSlaves should be a function');
      QUnit.strictEqual(typeof shop.setSlaves         , 'function', 'new Shop().setSlaves should be a function');
      QUnit.strictEqual(typeof shop.addSlave          , 'function', 'new Shop().addSlave should be a function');
      QUnit.strictEqual(typeof shop.removeSlaveByIndex, 'function', 'new Shop().removeSlaveByIndex should be a function');
    }
    {
      Shop = ArrayCapsule('employees', {
        methodNames: {
          'get' : 'getSlaves'
        }
      }).extend('Shop', {});
      shop = new Shop();
      QUnit.ok(!('getEmployees'  in shop), 'new Shop() should not have a method "getEmployees"');
      QUnit.ok('getSlaves'  in shop, 'new Shop() should have a method "getSlaves"');
      QUnit.strictEqual(typeof shop.getSlaves , 'function', 'new Shop().getSlaves should be a function');
    }
    {
      Shop = ArrayCapsule('employees', {
        methodNames: {
          'set' : 'setSlaves'
        }
      }).extend('Shop', {});
      shop = new Shop();
      QUnit.ok(!('setEmployees'  in shop), 'new Shop() should not have a method "setEmployees"');
      QUnit.ok('setSlaves'  in shop, 'new Shop() should have a method "setSlaves"');
      QUnit.strictEqual(typeof shop.setSlaves , 'function', 'new Shop().setSlaves should be a function');
    }
    {
      Shop = ArrayCapsule('employees', {
        methodNames: {
          'init': 'initSlaves'
        }
      }).extend('Shop', {});
      shop = new Shop();
      QUnit.ok(!('initEmployees' in shop), 'new Shop() should not have a method "initEmployees"');
      QUnit.ok('initSlaves' in shop, 'new Shop() should  have a method "initSlaves"');
      QUnit.strictEqual(typeof shop.initSlaves, 'function', 'new Shop().initSlaves should be a function');
    }
    {
      Shop = ArrayCapsule('employees', {
        methodNames: {
          'add': 'addSlave'
        }
      }).extend('Shop', {});
      shop = new Shop();
      QUnit.ok(!('addEmployees' in shop), 'new Shop() should not have a method "addEmployees"');
      QUnit.ok('addSlave' in shop, 'new Shop() should  have a method "addSlave"');
      QUnit.strictEqual(typeof shop.addSlave, 'function', 'new Shop().addSlave should be a function');
    }
    {
      Shop = ArrayCapsule('employees', {
        methodNames: {
          'removeByIndex': 'removeSlaveByIndex'
        }
      }).extend('Shop', {});
      shop = new Shop();
      QUnit.ok(!('removeByIndexEmployees' in shop), 'new Shop() should not have a method "removeByIndexEmployees"');
      QUnit.ok('removeSlaveByIndex' in shop, 'new Shop() should  have a method "removeSlaveByIndex"');
      QUnit.strictEqual(typeof shop.removeSlaveByIndex, 'function', 'new Shop().removeSlaveByIndex should be a function');
    }
  });
  /*
  QUnit.test('Options: validators', function() {
    var Shop;

    // Validators return a boolean
    {
      Shop = Initializer.extend('Shop', [ArrayCapsule('employees', {
        validators: [
            function(value) { return value === undefined || value >= 1; },
            function(value) { return value === undefined || value >= 2; }
          ]
      })], {});
      QUnit.raises(function() { new Shop().setStatus(0); }, ArrayCapsule.InvalidValueError, "new Shop().setStatus(0) shouldn't pass the first validator");
      QUnit.raises(function() { new Shop().setStatus(1); }, ArrayCapsule.InvalidValueError, "new Shop().setStatus(1) shouldn't pass the second validator");
      QUnit.ok(new Shop().setStatus(2) && true, "new Shop().setStatus(2) should pass all the filters");
    }

    // Validators throw exception
    {
      var EmploymentError1 = function EmploymentError1() {};
      var EmploymentError2 = function EmploymentError2() {};
      Shop = Initializer.extend('Shop', [ArrayCapsule('employees', {
        validators: [
            function(value) { if(value !== undefined && value < 1) throw new EmploymentError1(); return true; },
            function(value) { if(value !== undefined && value < 2) throw new EmploymentError2(); return true; }
          ]
      })], {});
      QUnit.raises(function() { new Shop().setStatus(0); }, EmploymentError1, "new Shop().setStatus(0) shouldn't pass the first validator");
      QUnit.raises(function() { new Shop().setStatus(1); }, EmploymentError2, "new Shop().setStatus(1) shouldn't pass the second validator");
      QUnit.ok(new Shop().setStatus(2) && true, "new Shop().setStatus(2) should pass all the filters");
    }
  });*/
});
