define(['jssl/core/ClassDebug', 'jssl/core/Initializer', 'jssl/mixins/Capsule'],
    function(Class, Initializer, Capsule) {

  QUnit.module('jssl/mixins/Capsule');

  QUnit.test('Instanciation', function() {
    var Task, task;
    {
      Task = Capsule('status').extend('Task', {});
      task = new Task();
      QUnit.ok('initStatus' in task, 'new Task() should have a method "initStatus"');
      QUnit.ok('getStatus' in task , 'new Task() should have a method "getStatus"');
      QUnit.ok('setStatus' in task , 'new Task() should have a method "setStatus"');
      QUnit.strictEqual(typeof task.initStatus, 'function', 'new Task().initStatus should be a function');
      QUnit.strictEqual(typeof task.getStatus , 'function', 'new Task().getStatus should be a function');
      QUnit.strictEqual(typeof task.setStatus , 'function', 'new Task().setStatus should be a function');
    }
    {
      Task = Initializer.extend('Task', [Capsule('status')], {});
      task = new Task();
      QUnit.ok('initStatus' in task, 'new Task() should have a method "initStatus"');
      QUnit.ok('getStatus' in task , 'new Task() should have a method "getStatus"');
      QUnit.ok('setStatus' in task , 'new Task() should have a method "setStatus"');
      QUnit.strictEqual(typeof task.initStatus, 'function', 'new Task().initStatus should be a function');
      QUnit.strictEqual(typeof task.getStatus , 'function', 'new Task().getStatus should be a function');
      QUnit.strictEqual(typeof task.setStatus , 'function', 'new Task().setStatus should be a function');
    }
  });

  QUnit.test('initXXX', function() {
    var Task = Capsule('status').extend('Task', {});
    var task = new Task();
    QUnit.ok('status' in task, 'new Task() should have a field "status"');
    QUnit.strictEqual(task.status, undefined, 'new Task().status should equal undefined');
  });

  QUnit.test('getXXX', function() {
    var Task = Capsule('status').extend('Task', {});
    var task = new Task();
    QUnit.strictEqual(task.getStatus(), undefined, 'new Task().getStatus() should equal undefined');
    task.status = "loading";
    QUnit.strictEqual(task.getStatus(), "loading", 'task.getStatus() should equal "loading"');
  });

  QUnit.test('setXXX', function() {
    var Task = Capsule('status').extend('Task', {});
    var task = new Task();
    QUnit.strictEqual(task.getStatus(), undefined, 'new Task().status should equal undefined');
    task.setStatus("loading");
    QUnit.strictEqual(task.getStatus(), "loading", 'task.getStatus() should equal "loading"');
    QUnit.strictEqual(task.setStatus("done"), task, 'task.setStatus() should return task');
    QUnit.strictEqual(task.getStatus(), "done", 'task.getStatus() should equal "done"');
  });


  QUnit.test('Options: default value', function() {

    var validValues = [undefined, null, 'ok'],
        i,
        Task,
        task;

    // Capsule as a parent
    for(i = 0 ; i < validValues.length ; i++) {
      Task = Capsule('status', {defaultValue:validValues[i]}).extend('Task', {});
      task = new Task();
      QUnit.strictEqual(task.status, validValues[i], '[parent] new Task().status should have default value ' + validValues[i]);
    }

    // Initializer as a parent and Capsule as a mixin
    for(i = 0 ; i < validValues.length ; i++) {
      Task = Initializer.extend('Task', [Capsule('status', {defaultValue:validValues[i]})], {});
      task = new Task();
      QUnit.strictEqual(task.status, validValues[i], '[mixin] new Task().status should have default value ' + validValues[i]);
    }
  });

  QUnit.test('Options: method names', function() {
    var Task, task;
    {
      Task = new Capsule('status', {
        methodNames: {
          'get' : 'getState',
          'set' : 'setState',
          'init': 'initState'
        }
      }).extend('Task', {});
      task = new Task();
      QUnit.ok(!('initStatus' in task), 'new Task() should not have a method "initStatus"');
      QUnit.ok(!('getStatus'  in task), 'new Task() should not have a method "getStatus"');
      QUnit.ok(!('setStatus'  in task), 'new Task() should not have a method "setStatus"');
      QUnit.ok('initState' in task, 'new Task() should  have a method "initState"');
      QUnit.ok('getState'  in task, 'new Task() should have a method "getState"');
      QUnit.ok('setState'  in task, 'new Task() should have a method "setState"');
      QUnit.strictEqual(typeof task.initState, 'function', 'new Task().initState should be a function');
      QUnit.strictEqual(typeof task.getState , 'function', 'new Task().getState should be a function');
      QUnit.strictEqual(typeof task.setState , 'function', 'new Task().setState should be a function');
    }
    {
      Task = new Capsule('status', {
        methodNames: {
          'get' : 'getState'
        }
      }).extend('Task', {});
      task = new Task();
      QUnit.ok(!('getStatus'  in task), 'new Task() should not have a method "getStatus"');
      QUnit.ok('getState'  in task, 'new Task() should have a method "getState"');
      QUnit.strictEqual(typeof task.getState , 'function', 'new Task().getState should be a function');
    }
    {
      Task = new Capsule('status', {
        methodNames: {
          'set' : 'setState'
        }
      }).extend('Task', {});
      task = new Task();
      QUnit.ok(!('setStatus'  in task), 'new Task() should not have a method "setStatus"');
      QUnit.ok('setState'  in task, 'new Task() should have a method "setState"');
      QUnit.strictEqual(typeof task.setState , 'function', 'new Task().setState should be a function');
    }
    {
      Task = new Capsule('status', {
        methodNames: {
          'init': 'initState'
        }
      }).extend('Task', {});
      task = new Task();
      QUnit.ok(!('initStatus' in task), 'new Task() should not have a method "initStatus"');
      QUnit.ok('initState' in task, 'new Task() should  have a method "initState"');
      QUnit.strictEqual(typeof task.initState, 'function', 'new Task().initState should be a function');
    }
  });

  QUnit.test('Options: validators', function() {
    var Task;

    // Validators return a boolean
    {
      Task = Initializer.extend('Task', [Capsule('status', {
        validators: [
            function(value) { return value === undefined || value >= 1; },
            function(value) { return value === undefined || value >= 2; }
          ]
      })], {});
      QUnit.raises(function() { new Task().setStatus(0); }, Capsule.InvalidValueError, "new Task().setStatus(0) shouldn't pass the first validator");
      QUnit.raises(function() { new Task().setStatus(1); }, Capsule.InvalidValueError, "new Task().setStatus(1) shouldn't pass the second validator");
      QUnit.ok(new Task().setStatus(2) && true, "new Task().setStatus(2) should pass all the filters");
    }

    // Validators throw exception
    {
      var TaskError1 = function TaskError1() {};
      var TaskError2 = function TaskError2() {};
      Task = Initializer.extend('Task', [Capsule('status', {
        validators: [
            function(value) { if(value !== undefined && value < 1) throw new TaskError1(); return true; },
            function(value) { if(value !== undefined && value < 2) throw new TaskError2(); return true; }
          ]
      })], {});
      QUnit.raises(function() { new Task().setStatus(0); }, TaskError1, "new Task().setStatus(0) shouldn't pass the first validator");
      QUnit.raises(function() { new Task().setStatus(1); }, TaskError2, "new Task().setStatus(1) shouldn't pass the second validator");
      QUnit.ok(new Task().setStatus(2) && true, "new Task().setStatus(2) should pass all the filters");
    }
  });
});
