define([
  'jssl/network/ipv4/Address'
], function (Address) {
  'use strict';

  QUnit.module('jssl/network/ipv4/Address');

  QUnit.test("new Address()", function() {
    QUnit.expect(6);

    var address1 = new Address('192.168.0.1');
    var address2 = new Address('255.255.255.0');
    var address3 = new Address('0.0.0.0');
    var address4 = new Address('255.255.255.255');

    QUnit.ok(address1 instanceof Address, "new Address('192.168.0.1') instanceof Address");
    QUnit.ok(address3 instanceof Address, "new Address('0.0.0.0') instanceof Address");
    QUnit.ok(address4 instanceof Address, "new Address('255.255.255.255') instanceof Address");

    QUnit.strictEqual(address2.address, -256, "new Address('255.255.255.0').address === -256");
    QUnit.strictEqual(address3.address, 0, "new Address('0.0.0.0').address === 0");
    QUnit.strictEqual(address4.address, -1, "new Address('255.255.255.255').address === -1");
  });


  QUnit.test('Address.toString()', function() {
    QUnit.expect(4);

    var address1 = new Address('0.0.0.0');
    var address2 = new Address('255.255.255.0');
    var address3 = new Address('255.255.255.255');
    var address4 = new Address('192.168.0.1');

    QUnit.strictEqual(address1.toString(), '0.0.0.0'        , "new Address('0.0.0.0').toString() === '0.0.0.0'");
    QUnit.strictEqual(address2.toString(), '255.255.255.0'  , "new Address('255.255.255.0').toString() === '255.255.255.0'");
    QUnit.strictEqual(address3.toString(), '255.255.255.255', "new Address('255.255.255.255').toString() === '255.255.255.255'");
    QUnit.strictEqual(address4.toString(), '192.168.0.1'    , "new Address('192.168.0.1').toString() === '192.168.0.1'");
  });


  QUnit.test('Address.getNext()', function() {
    QUnit.expect(6);

    var address1a = new Address('0.0.0.0');
    var address1b = new Address('0.0.0.1');

    var address2a = new Address('192.168.10.100');
    var address2b = new Address('192.168.10.101');

    var address3a = new Address('192.168.0.255');
    var address3b = new Address('192.168.1.0');

    var address4a = new Address('192.255.255.255');
    var address4b = new Address('193.0.0.0');

    var address5a = new Address('127.255.255.255');
    var address5b = new Address('128.0.0.0');

    var address6 = new Address('255.255.255.255');

    QUnit.deepEqual(address1a.getNext(), address1b, "new Address('0.0.0.0').getNext() === '0.0.0.1'");
    QUnit.deepEqual(address2a.getNext(), address2b, "new Address('192.168.10.100').getNext() === '192.168.10.101'");
    QUnit.deepEqual(address3a.getNext(), address3b, "new Address('192.168.0.255').getNext() === '192.168.1.0'");
    QUnit.deepEqual(address4a.getNext(), address4b, "new Address('192.255.255.255').getNext() === '193.0.0.0'");
    QUnit.deepEqual(address5a.getNext(), address5b, "new Address('127.255.255.255').getNext() === '128.0.0.0'");
    QUnit.raises(address6.getNext, "new Address('255.255.255.255').getNext() must throw an error");
  });


  QUnit.test('Address.getPrevious()', function() {
    QUnit.expect(6);

    var address1a = new Address('255.255.255.255');
    var address1b = new Address('255.255.255.254');

    var address2a = new Address('192.168.10.100');
    var address2b = new Address('192.168.10.99');

    var address3a = new Address('192.168.1.0');
    var address3b = new Address('192.168.0.255');

    var address4a = new Address('193.0.0.0');
    var address4b = new Address('192.255.255.255');

    var address5a = new Address('128.0.0.0');
    var address5b = new Address('127.255.255.255');

    var address6 = new Address('0.0.0.0');

    QUnit.deepEqual(address1a.getPrevious(), address1b, "new Address('255.255.255.255').getPrevious() === '255.255.255.254'");
    QUnit.deepEqual(address2a.getPrevious(), address2b, "new Address('192.168.10.100').getPrevious() === '192.168.10.99'");
    QUnit.deepEqual(address3a.getPrevious(), address3b, "new Address('192.168.1.0').getPrevious() === '192.168.0.255'");
    QUnit.deepEqual(address4a.getPrevious(), address4b, "new Address('193.0.0.0').getPrevious() === '192.255.255.255'");
    QUnit.deepEqual(address5a.getPrevious(), address5b, "new Address('128.0.0.0').getPrevious() === '127.255.255.255'");
    QUnit.raises(address6.getPrevious, "new Address('0.0.0.0').getPrevious() must throw an error");
  });


});
