define([
  'jssl/network/ipv4/Address',
  'jssl/network/ipv4/Mask'
], function (Address, Mask) {
  'use strict';

  QUnit.module('jssl/network/ipv4/Mask');

  QUnit.test("new Mask()", function() {
    QUnit.expect(18 + 33);

    var masks = [], i;

    for(i = 0 ; i <= 32 ; i++) {
      masks[i] = new Mask(i);
    }

    var mask11 = new Mask(24);
    var mask12 = new Mask('24');
    var mask13 = new Mask('255.255.255.0');

    var mask21 = new Mask(32);
    var mask22 = new Mask('32');
    var mask23 = new Mask('255.255.255.255');

    var mask31 = new Mask(0);
    var mask32 = new Mask('0');
    var mask33 = new Mask('0.0.0.0');

    QUnit.ok(mask11 instanceof Mask, "new Mask(24) instanceof Mask");
    QUnit.ok(mask12 instanceof Mask, "new Mask('24') instanceof Mask");
    QUnit.ok(mask13 instanceof Mask, "new Mask('255.255.255.0') instanceof Mask");

    QUnit.ok(mask21 instanceof Mask, "new Mask(32) instanceof Mask");
    QUnit.ok(mask22 instanceof Mask, "new Mask('32') instanceof Mask");
    QUnit.ok(mask23 instanceof Mask, "new Mask('255.255.255.255') instanceof Mask");

    QUnit.ok(mask31 instanceof Mask, "new Mask(0) instanceof Mask");
    QUnit.ok(mask32 instanceof Mask, "new Mask('0') instanceof Mask");
    QUnit.ok(mask33 instanceof Mask, "new Mask('0.0.0.0') instanceof Mask");

    for(i = 0 ; i <= 32 ; i++) {
      QUnit.strictEqual(masks[i].mask, i === 0 ? 0 : -1 << (32 - i), 'new Mask(' + i + ').mask === -1 << ' + (32 - i) + ' === ' + (i === 0 ? 0 : -1 << (32 - i)));
    }

    QUnit.deepEqual(mask12, mask11, "new Mask('24') === new Mask(24)");
    QUnit.deepEqual(mask13, mask11, "new Mask('255.255.255.0') === new Mask(24)");

    QUnit.deepEqual(mask22, mask21, "new Mask('32') === new Mask(32)");
    QUnit.deepEqual(mask23, mask21, "new Mask('255.255.255.255') === new Mask(32)");

    QUnit.deepEqual(mask32, mask31, "new Mask('0') === new Mask(0)");
    QUnit.deepEqual(mask33, mask31, "new Mask('0.0.0.0') === new Mask(0)");

    QUnit.strictEqual(mask13.mask, -256, "new Mask('255.255.255.0').mask === -256");
    QUnit.strictEqual(mask23.mask, -1  , "new Mask('255.255.255.255').mask === -1");
    QUnit.strictEqual(mask33.mask, 0   , "new Mask('0.0.0.0').mask === 0");
  });


  QUnit.test('Mask.toString()', function() {
    QUnit.expect(4);

    var mask1 = new Mask('255.255.255.0');
    var mask2 = new Mask('0.0.0.0');
    var mask3 = new Mask('255.255.255.255');
    var mask4 = new Mask('192.168.0.14');

    QUnit.strictEqual(mask1.toString(), '255.255.255.0', "new Mask('255.255.255.0').toString() === '255.255.255.0'");
    QUnit.strictEqual(mask2.toString(), '0.0.0.0', "new Mask('0.0.0.0').toString() === '0.0.0.0'");
    QUnit.strictEqual(mask3.toString(), '255.255.255.255', "new Mask('255.255.255.255').toString() === '255.255.255.255'");
    QUnit.strictEqual(mask4.toString(), '192.168.0.14', "new Mask('192.168.0.14').toString() === '192.168.0.14'");
  });


  QUnit.test('Mask.getMaskSize()', function() {
    QUnit.expect(33);

    var masks = [], i;

    for(i = 0 ; i <= 32 ; i++) {
      masks[i] = new Mask(i);
    }

    for(i = 0 ; i <= 32 ; i++) {
      QUnit.strictEqual(masks[i].getMaskSize(), i, 'new Mask(' + i + ').getMaskSize() === ' + i);
    }
  });


  QUnit.test('Mask.apply()', function() {
    QUnit.expect(4);

    var mask1    = new Mask(16);
    var address1 = new Address('192.168.0.0');

    QUnit.deepEqual(mask1.apply('192.168.0.1'), address1, "new Mask(16).apply('192.168.0.1') === '192.168.0.0'");
    QUnit.deepEqual(mask1.apply('192.168.0.255'), address1, "new Mask(16).apply('192.168.0.255') === '192.168.0.0'");
    QUnit.deepEqual(mask1.apply('192.168.1.0'), address1, "new Mask(16).apply('192.168.1.0') === '192.168.0.0'");
    QUnit.deepEqual(mask1.apply('192.168.1.254'), address1, "new Mask(16).apply('192.168.1.254') === '192.168.0.0'");
  });

});
