define([
  'jssl/network/ipv4/Address',
  'jssl/network/ipv4/Subnet',
  'jssl/network/ipv4/Pool'
], function (Address, Subnet, Pool) {
  'use strict';

  QUnit.module('jssl/network/ipv4/Pool');

  QUnit.test("Subnet.prototype.getFirstAvailableAddress()", function() {

    var address1 = new Address('192.168.0.1');
    var address2 = new Address('192.168.0.2');
    var address3 = new Address('192.168.0.3');
    var address4 = new Address('192.168.0.4');
    var address5 = new Address('192.168.0.5');
    var address6 = new Address('192.168.0.6');

    var pool1 = new Pool(new Subnet('192.168.0.0', 29));

    var pool2 = new Pool(new Subnet('192.168.0.0', 29));
    pool2
      .allocate(address1)
      .allocate(address2)
      .allocate(address4);

    var pool3 = new Pool(new Subnet('192.168.0.0', 29));
    pool3.allocate([address1, address2, address3, address4, address5, address6]);

    QUnit.deepEqual(pool1.getFirstAvailable(), address1, "new Pool(new Subnet('192.168.0.0', 24)).getFirstAvailable() === '192.168.0.1'");
    QUnit.deepEqual(pool2.getFirstAvailable(), address3, "new Pool(new Subnet('192.168.0.0', 24)).allocate(...).getFirstAvailable() === '192.168.0.3'");
    QUnit.strictEqual(pool3.getFirstAvailable(), null, "new Pool(new Subnet('192.168.0.0', 24)).allocate(*).getFirstAvailable() === null");
  });

  QUnit.test("Subnet.prototype.getLastAvailableAddress()", function() {

    var address1 = new Address('192.168.0.1');
    var address2 = new Address('192.168.0.2');
    var address3 = new Address('192.168.0.3');
    var address4 = new Address('192.168.0.4');
    var address5 = new Address('192.168.0.5');
    var address6 = new Address('192.168.0.6');

    var pool1 = new Pool(new Subnet('192.168.0.0', 29));

    var pool2 = new Pool(new Subnet('192.168.0.0', 29));
    pool2
      .allocate(address6)
      .allocate(address5)
      .allocate(address3);

    var pool3 = new Pool(new Subnet('192.168.0.0', 29));
    pool3.allocate([address1, address2, address3, address4, address5, address6]);

    QUnit.deepEqual(pool1.getLastAvailable(), address6, "new Pool(new Subnet('192.168.0.0', 24)).getLastAvailable() === '192.168.0.6'");
    QUnit.deepEqual(pool2.getLastAvailable(), address4, "new Pool(new Subnet('192.168.0.0', 24)).allocate(...).getLastAvailable() === '192.168.0.4'");
    QUnit.strictEqual(pool3.getLastAvailable(), null, "new Pool(new Subnet('192.168.0.0', 24)).allocate(*).getLastAvailable() === null");

  });

});
