define([
  'jssl/network/ipv4/Address',
  'jssl/network/ipv4/Mask',
  'jssl/network/ipv4/Subnet'
], function (Address, Mask, Subnet) {
  'use strict';

  QUnit.module('jssl/network/ipv4/Subnet');

  test("new Subnet()", function() {
    QUnit.expect(13);

    var subnet1 = new Subnet('192.168.1.1', 32);
    var subnet2 = new Subnet('192.168.1.0', 24);
    var subnet3 = new Subnet('192.168.0.0', 16);
    var subnet4 = new Subnet('0.0.0.0', 0);

    QUnit.ok(subnet1 instanceof Subnet, "new Subnet('192.168.1.1', 32) instanceof Subnet");
    QUnit.ok(subnet2 instanceof Subnet, "new Subnet('192.168.1.0', 24) instanceof Subnet");
    QUnit.ok(subnet3 instanceof Subnet, "new Subnet('192.168.0.0', 16) instanceof Subnet");
    QUnit.ok(subnet4 instanceof Subnet, "new Subnet('0.0.0.0', 32) instanceof Subnet");

    QUnit.deepEqual(subnet1.network, new Address('192.168.1.1'), "new Subnet('192.168.1.1', 32).network === new Address('192.168.1.1')");
    QUnit.deepEqual(subnet2.network, new Address('192.168.1.0'), "new Subnet('192.168.1.0', 24).network === new Address('192.168.1.0')");
    QUnit.deepEqual(subnet3.network, new Address('192.168.0.0'), "new Subnet('192.168.0.0', 16).network === new Address('192.168.0.0')");
    QUnit.deepEqual(subnet4.network, new Address('0.0.0.0'), "new Subnet('192.168.1.1', 32).network === new Address('192.168.1.1')");

    QUnit.deepEqual(subnet1.mask, new Mask('255.255.255.255'), "new Subnet('192.168.1.1', 32).mask === new Mask('255.255.255.255')");
    QUnit.deepEqual(subnet2.mask, new Mask('255.255.255.0'), "new Subnet('192.168.1.0', 24).mask === new Mask('255.255.255.0')");
    QUnit.deepEqual(subnet3.mask, new Mask('255.255.0.0'), "new Subnet('192.168.0.0', 16).mask === new Mask('255.255.0.0')");
    QUnit.deepEqual(subnet4.mask, new Mask('0.0.0.0'), "new Subnet('0.0.0.0', 0).mask === new Mask('0.0.0.0')");

    QUnit.raises(
      function() {
        new Subnet('192.168.1.0', '255.255.0.0');
      },
      'new Subnet(\'192.168.1.0\', \'255.255.0.0\') throws \'Invalid IPv4 Subnet "192.168.1.0/16", should be "192.168.0.0/16".\'');
  });


  test("isValidAddress()", function() {
    QUnit.expect(7);

    var subnet1 = new Subnet('192.168.0.0', 16);

    QUnit.ok(subnet1.isValidAddress('192.168.0.1'), "new Subnet('192.168.0.0', 16).isValidAddress('192.168.0.1') === true");
    QUnit.ok(subnet1.isValidAddress('192.168.0.255'), "new Subnet('192.168.0.0', 16).isValidAddress('192.168.0.255') === true");
    QUnit.ok(subnet1.isValidAddress('192.168.1.0'), "new Subnet('192.168.0.0', 16).isValidAddress('192.168.1.0') === true");

    QUnit.ok(!subnet1.isValidAddress('192.168.0.0'), "new Subnet('192.168.0.0', 16).isValidAddress('192.168.0.0') === false");
    QUnit.ok(!subnet1.isValidAddress('192.168.255.255'), "new Subnet('192.168.255.255', 16).isValidAddress('192.168.255.255') === false");
    QUnit.ok(!subnet1.isValidAddress('192.167.255.255'), "new Subnet('192.168.0.0', 16).isValidAddress('192.167.255.255') === false");
    QUnit.ok(!subnet1.isValidAddress('192.169.0.0'), "new Subnet('192.168.0.0', 16).isValidAddress('192.169.0.0') === false");
  });

});
